==> on_upgrade-0.1.0.tar.gz: lib/on_upgrade.dart <==
/// Provides the ability to check if the currently started app is an upgrade.
///
/// An upgrade is defined as the first start of a new app version, compared to the last known version.
/// All versions should follow the [Dart package versioning guidelines](https://dart.dev/tools/pub/versioning).
///
/// Can be used with the given persistence implementation (shared preferences) or with a custom implementation of the getter / setter logic
/// for the last known version. The currently running app version is always determined by the version loaded via
/// [package_info_plus](https://pub.dev/packages/package_info_plus).
library on_upgrade;

import 'package:flutter/widgets.dart';
import 'package:package_info_plus/package_info_plus.dart';
import 'package:shared_preferences/shared_preferences.dart';

enum UpgradeState {
  upgrade,
  noUpgrade,
  unknown,
}

/// Wraps the upgrade state of the app.
///
/// If [hasError] is **false** the upgrade check was successful and [state], [lastVersion] and [currentVersion] are set accordingly.
///
/// If an error occurred [hasError] is **true**, [state] is [UpgradeState.unknown] and the error reason is returned in [error].
class UpgradeWrapper {
  final UpgradeState state;
  final String lastVersion;
  final String currentVersion;
  final bool hasError;
  final Exception error;

  UpgradeWrapper(
      {@required this.state,
      this.lastVersion,
      this.currentVersion,
      this.hasError = false,
      this.error});
}

/// Contains the upgrade check functionality
///
/// The last know version must be set via [updateLastVersion].
///
/// Usage with default implementation:
/// ```dart
/// final onUpgrade = OnUpgrade();
/// final isNewVersion = await onUpgrade.isNewVersion();
/// if (isNewVersion.isUpdate == UpgradeState.upgrade) {
///   await onUpgrade.updateLastVersion();
///   myDataMigration();
///   myShowUserNewFeaturesDialog();
/// }
/// ```
class OnUpgrade {
  static const _fallbackVersion = '0.0.0';

  /// Configures a custom key used in the shared preferences. Defaults to `on_upgrade.version`
  final String keyLastVersion;

  /// Optional current version getter. If set [keyLastVersion] isn't used and [customVersionUpdate] is required to be also set.
  /// Must have the signature `Future<String> func() async {}` and must return an empty string if no last version is given during initial start.
  final Function customVersionLookup;

  /// Optional current version setter. If set [keyLastVersion] isn't used and [customVersionLookup] is required to be also set
  /// Must have the signature `Future<bool> func([String version]) async {}`
  final Function customVersionUpdate;

  OnUpgrade(
      {this.keyLastVersion = 'on_upgrade.version',
      this.customVersionLookup,
      this.customVersionUpdate});

  /// Returns the upgrade state.
  Future<UpgradeWrapper> isNewVersion() async {
    try {
      final lastVersion = await getLastVersion();
      final currentVersion = await getCurrentVersion();
      final state = _checkNewVersion(lastVersion, currentVersion)
          ? UpgradeState.upgrade
          : UpgradeState.noUpgrade;
      return UpgradeWrapper(
          state: state,
          lastVersion: lastVersion,
          currentVersion: currentVersion);
    } on Exception catch (exception) {
      return UpgradeWrapper(
          state: UpgradeState.unknown, hasError: true, error: exception);
    }
  }

  /// Returns if the current app start is the first start overall.
  Future<bool> isInitialInstallation() async {
    final version = await _getLastVersionString();
    return version != null && version.isEmpty;
  }

  /// Returns the last known version or '0.0.0' as initial fallback value. Call [updateLastVersion] to update this value
  Future<String> getLastVersion() async {
    final version = await _getLastVersionString();
    return version != null && version.isNotEmpty ? version : _fallbackVersion;
  }

  /// Returns the currently running version
  Future<String> getCurrentVersion() async {
    final packageInfo = await PackageInfo.fromPlatform();
    return packageInfo.version;
  }

  /// Returns the currently running version as [String]
  Future<String> getCurrentVersionString() async {
    return (await getCurrentVersion()).toString();
  }

  /// Sets the last know version and persists it. If called without parameter the currently running version is set.
  Future<bool> updateLastVersion([String version]) async {
    if (customVersionUpdate != null) {
      return customVersionUpdate(version);
    }
    version ??= await getCurrentVersionString();
    final prefs = await SharedPreferences.getInstance();
    return prefs.setString(keyLastVersion, version);
  }

  Future<String> _getLastVersionString() async {
    if (customVersionLookup != null) {
      return customVersionLookup();
    }
    final prefs = await SharedPreferences.getInstance();
    return prefs.getString(keyLastVersion) ?? '';
  }

  bool _checkNewVersion(String lastVersion, String currentVersion) {
    if (lastVersion == null || currentVersion == null) {
      throw FormatException(
          'Both lastVersion and currentVersion must be non null');
    }
    if (currentVersion.isEmpty) {
      throw FormatException("Couldn't load currentVersion");
    }
    return lastVersion != currentVersion;
  }
}

==> on_upgrade-1.0.0.tar.gz: lib/on_upgrade.dart <==
/// Provides the ability to check if the currently started app is an upgrade.
///
/// An upgrade is defined as the first start of a new app version, compared to the last known version.
/// All versions should follow the [Dart package versioning guidelines](https://dart.dev/tools/pub/versioning).
///
/// Can be used with the given persistence implementation (shared preferences) or with a custom implementation of the getter / setter logic
/// for the last known version. The currently running app version is always determined by the version loaded via
/// [package_info_plus](https://pub.dev/packages/package_info_plus).
library on_upgrade;

import 'package:package_info_plus/package_info_plus.dart';
import 'package:shared_preferences/shared_preferences.dart';

enum UpgradeState {
  upgrade,
  noUpgrade,
  unknown,
}

/// Wraps the upgrade state of the app.
///
/// If [hasError] is **false** the upgrade check was successful and [state], [lastVersion] and [currentVersion] are set accordingly.
///
/// If an error occurred [hasError] is **true**, [state] is [UpgradeState.unknown] and the error reason is returned in [error].
class UpgradeWrapper {
  final UpgradeState state;
  final String? lastVersion;
  final String? currentVersion;
  final bool hasError;
  final Exception? error;

  UpgradeWrapper(
      {required this.state,
      this.lastVersion,
      this.currentVersion,
      this.hasError = false,
      this.error});
}

/// Custom version lookup with the signature `Future<String> func() async {}`. Must return an empty string if no last version is given during initial start.
typedef VersionLookup = Future<String> Function();

/// Custom version update with the signature `Future<bool> func([String version]) async {}`. Use `OnUpgrade().getCurrentVersionString` to get the currently running app version.
typedef VersionUpdate = Future<bool> Function([String? version]);

/// Contains the upgrade check functionality
///
/// The last know version must be set via [updateLastVersion].
///
/// Usage with default implementation:
/// ```dart
/// final onUpgrade = OnUpgrade();
/// final isNewVersion = await onUpgrade.isNewVersion();
/// if (isNewVersion.isUpdate == UpgradeState.upgrade) {
///   await onUpgrade.updateLastVersion();
///   myDataMigration();
///   myShowUserNewFeaturesDialog();
/// }
/// ```
class OnUpgrade {
  static const _fallbackVersion = '0.0.0';

  /// Configures a custom key used in the shared preferences. Defaults to `on_upgrade.version`
  final String keyLastVersion;

  /// Optional last version getter. If set [keyLastVersion] isn't used and [customVersionUpdate] is required to be also set.
  final VersionLookup? customVersionLookup;

  /// Optional last version setter. If set [keyLastVersion] isn't used and [customVersionLookup] is required to be also set
  final VersionUpdate? customVersionUpdate;

  OnUpgrade(
      {this.keyLastVersion = 'on_upgrade.version',
      this.customVersionLookup,
      this.customVersionUpdate});

  /// Returns the upgrade state.
  Future<UpgradeWrapper> isNewVersion() async {
    try {
      final lastVersion = await getLastVersion();
      final currentVersion = await getCurrentVersion();
      final state = _checkNewVersion(lastVersion, currentVersion)
          ? UpgradeState.upgrade
          : UpgradeState.noUpgrade;
      return UpgradeWrapper(
          state: state,
          lastVersion: lastVersion,
          currentVersion: currentVersion);
    } on Exception catch (exception) {
      return UpgradeWrapper(
          state: UpgradeState.unknown, hasError: true, error: exception);
    }
  }

  /// Returns if the current app start is the first start overall.
  Future<bool> isInitialInstallation() async {
    final version = await _getLastVersionString();
    return version.isEmpty;
  }

  /// Returns the last known version or '0.0.0' as initial fallback value. Call [updateLastVersion] to update this value
  Future<String> getLastVersion() async {
    final version = await _getLastVersionString();
    return version.isNotEmpty ? version : _fallbackVersion;
  }

  /// Returns the currently running version
  Future<String> getCurrentVersion() async {
    final packageInfo = await PackageInfo.fromPlatform();
    return packageInfo.version;
  }

  /// Sets the last know version and persists it. If called without parameter the currently running version is set.
  Future<bool> updateLastVersion([String? version]) async {
    if (customVersionUpdate != null) {
      return customVersionUpdate!(version);
    }
    version ??= await getCurrentVersion();
    final prefs = await SharedPreferences.getInstance();
    return prefs.setString(keyLastVersion, version);
  }

  Future<String> _getLastVersionString() async {
    if (customVersionLookup != null) {
      return customVersionLookup!();
    }
    final prefs = await SharedPreferences.getInstance();
    return prefs.getString(keyLastVersion) ?? '';
  }

  bool _checkNewVersion(String lastVersion, String currentVersion) {
    if (currentVersion.isEmpty) {
      throw FormatException("Couldn't load currentVersion");
    }
    return lastVersion != currentVersion;
  }
}
